﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using VeterinaryContracts.BusinessLogicContracts;
using VeterinaryContracts.BindingModels;
using VeterinaryContracts.SearchModels;
using VeterinaryContracts.StorageContracts;
using VeterinaryContracts.ViewModels;
using Microsoft.Extensions.Logging;
using VeterinaryDataModels;

namespace VeterinaryBusinessLogic.BusinessLogic
{
	public class DrugLogic : IDrugLogic
	{
		private readonly ILogger _logger;
		private readonly IDrugStorage _drugStorage;
		public DrugLogic(ILogger<DrugLogic> logger, IDrugStorage drugStorage)
		{
			_logger = logger;
			_drugStorage = drugStorage;
		}
		public List<DrugViewModel>? ReadList(DrugSearchModel? model)
		{
			_logger.LogInformation("ReadList. DrugName:{DrugName}.Id:{ Id}", model?.DrugName, model?.Id);
			var list = model == null ? _drugStorage.GetFullList() : _drugStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count:{Count}", list.Count);
			return list;
		}
		public DrugViewModel? ReadElement(DrugSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. DrugName:{DrugName}.Id:{ Id}", model.DrugName, model.Id);
			var element = _drugStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
			return element;
		}
		public bool Create(DrugBindingModel model)
		{
			CheckModel(model);
			if (_drugStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}
		public bool Update(DrugBindingModel model)
		{
			CheckModel(model);
			if (_drugStorage.Update(model) == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			return true;
		}
		public bool Delete(DrugBindingModel model)
		{
			CheckModel(model, false);
			_logger.LogInformation("Delete. Id:{Id}", model.Id);
			if (_drugStorage.Delete(model) == null)
			{
				_logger.LogWarning("Delete operation failed");
				return false;
			}
			return true;
		}
		private void CheckModel(DrugBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			if (string.IsNullOrEmpty(model.DrugName))
			{
				throw new ArgumentNullException("Нет названия лекарства", nameof(model.DrugName));
			}
			if (model.Count <= 0)
			{
				throw new ArgumentNullException("Лекарств должно быть больше нуля", nameof(model.Count));
			}
			if (model.Price <= 0)
			{
				throw new ArgumentNullException("Цена лекарства должна быть больше 0", nameof(model.Price));
			}
			_logger.LogInformation("Drug. DrugName:{DrugName}.Count:{ Count}.Price:{ Price}. Id: { Id}", model.DrugName,model.Count, model.Price, model.Id);
			var element = _drugStorage.GetElement(new DrugSearchModel
			{
				DrugName = model.DrugName
			});
			if (element != null && element.Id != model.Id)
			{
				throw new InvalidOperationException("Лекарство с таким названием уже есть");
			}
		}
		public bool MakeSell(DrugBindingModel drug, int count)
		{
			return _drugStorage.SellDrugs(drug, count);
		}
	}
}
